﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace CreditDemo
{
    /// <summary>
    /// 类名：TripleDESHelper
    /// 功能：DES3加密、解密
    /// 详细：该类基于DES3进行加密和解密得到相应字符串
    /// 作者：钟伟鹏
    /// 修改时间：2018-07-30
    /// </summary>
    public class TripleDESHelper
    {
        /// <summary>
        /// 3DES加密
        /// step 1：string基于UTF-8格式转为byte[]
        /// step 2：创建TripleDESCryptoServiceProvider实例，赋值Key、Mode（与Java一直）、Padding（Java的PKCS5Padding与.Net的PaddingMode.PKCS7对应）、IV
        /// step 3：加密数据流获取加密后的byte[]，然后再基于base64格式转为string
        /// </summary>
        /// <param name="key"></param>
        /// <param name="data"></param>
        /// <returns></returns>
        public static string TripleDESEncrypt(string keyStr, string dataStr, string vectorStr)
        {
            byte[] key = Encoding.UTF8.GetBytes(keyStr);
            byte[] data = Encoding.UTF8.GetBytes(dataStr);
            byte[] vector = Encoding.UTF8.GetBytes(vectorStr);

            byte[] k = new byte[24];
            int len = data.Length;
            if (data.Length % 8 != 0)
            {
                len = data.Length - data.Length % 8 + 8;
            }
            byte[] needData = null;
            if (len != 0)
                needData = new byte[len];

            for (int i = 0; i < len; i++)
            {
                needData[i] = 0x00;
            }

            Buffer.BlockCopy(data, 0, needData, 0, data.Length);

            if (key.Length == 16)
            {
                Buffer.BlockCopy(key, 0, k, 0, key.Length);
                Buffer.BlockCopy(key, 0, k, 16, 8);
            }
            else
            {
                Buffer.BlockCopy(key, 0, k, 0, 24);
            }

            var des3 = new TripleDESCryptoServiceProvider();
            des3.Key = k;
            des3.Mode = CipherMode.CBC;
            des3.Padding = PaddingMode.PKCS7;
            des3.IV = vector;

            byte[] resultArray;
            using (MemoryStream ms = new MemoryStream())
            using (CryptoStream cs = new CryptoStream(ms, des3.CreateEncryptor(), CryptoStreamMode.Write))
            {
                cs.Write(data, 0, data.Length);
                cs.FlushFinalBlock();
                resultArray = ms.ToArray();
            }
            return Convert.ToBase64String(resultArray, 0, resultArray.Length);
        }
        /// <summary>
        /// 3DES解密
        /// step 1：string基于UTF-8格式转为byte[]，其中data基于base64格式转为byte[]
        /// step 2：创建TripleDESCryptoServiceProvider实例，赋值Key、Mode（与Java一直）、Padding（Java的PKCS5Padding与.Net的PaddingMode.PKCS7对应）、IV
        /// step 3：解密数据流获取解密后的byte[]，然后再基于UTF-8格式转为string
        /// </summary>
        /// <param name="keyStr"></param>
        /// <param name="dataStr"></param>
        /// <param name="vectorStr"></param>
        /// <returns></returns>
        public static string TripleDESDecrypt(string keyStr, string dataStr, string vectorStr)
        {
            byte[] key = Encoding.UTF8.GetBytes(keyStr);
            byte[] data = Convert.FromBase64String(dataStr);
            byte[] vector = Encoding.UTF8.GetBytes(vectorStr);

            byte[] k = new byte[24];
            int len = data.Length;
            if (data.Length % 8 != 0)
            {
                len = data.Length - data.Length % 8 + 8;
            }
            byte[] needData = null;
            if (len != 0)
                needData = new byte[len];

            for (int i = 0; i < len; i++)
            {
                needData[i] = 0x00;
            }

            Buffer.BlockCopy(data, 0, needData, 0, data.Length);

            if (key.Length == 16)
            {
                Buffer.BlockCopy(key, 0, k, 0, key.Length);
                Buffer.BlockCopy(key, 0, k, 16, 8);
            }
            else
            {
                Buffer.BlockCopy(key, 0, k, 0, 24);
            }

            var des3 = new TripleDESCryptoServiceProvider();
            des3.Key = k;
            des3.Mode = CipherMode.CBC;
            des3.Padding = PaddingMode.PKCS7;
            des3.IV = vector;
            byte[] resultArray;
            using (MemoryStream ms = new MemoryStream())
            using (CryptoStream cs = new CryptoStream(ms, des3.CreateDecryptor(), CryptoStreamMode.Write))
            {
                cs.Write(data, 0, data.Length);
                cs.FlushFinalBlock();
                resultArray = ms.ToArray();
            }
            return Encoding.UTF8.GetString(resultArray);
        }
    }
}